//
//  InsertJobViewController.m
//  iTimesheet
//
//  Created by Mark Brindle on 02/04/2011.
//  Copyright 2011 ARKEMM Software Limited. All rights reserved.
//

#import "InsertJobViewController.h"
#import "iTimesheetAppDelegate.h"
#import "PPProject.h"
#import "PPCostCentre.h"

@implementation InsertJobViewController

@synthesize defaultCostCentre;
@synthesize costCentrePicker;
@synthesize project;
@synthesize jobNameValue;
@synthesize jobDescValue;
@synthesize jobCostCentre;
@synthesize costCentres;

#pragma mark - boilerplate code (provided by template)

- (void)didReceiveMemoryWarning
{
  // Releases the view if it doesn't have a superview.
  [super didReceiveMemoryWarning];
  
  // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  // Return YES for supported orientations
  return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Existing methods (in the original template) that were amended

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
  // Just call init on ourself.  We are not concerned with the input parameters.
  return [self init];
}

- (void)viewDidLoad
{
  [super viewDidLoad];
  // Do any additional setup after loading the view from its nib.
  
  // If we have a project, show its name in the title
  if (project == nil) {
    [self setTitle:@"New Job"];
  } else {
    [self setTitle:[NSString stringWithFormat:@"%@ - New Job", [project name]]];
  }

  // Turn off autocorrect, so names will not be changed
  [jobName setAutocorrectionType:UITextAutocorrectionTypeNo];
  
  // If we have a cost centre then select it in the picker & set default label
  if (jobCostCentre != nil) {
    [[self costCentrePicker] selectRow:[[self costCentres]
                                        indexOfObject:jobCostCentre]
                           inComponent:kCostCentreComponent
                              animated:NO];
    [[self defaultCostCentre] 
     setText:[NSString stringWithFormat: @"(Project cost centre: %@)", 
              [jobCostCentre name]]];
  }

}

- (void)viewDidUnload
{
  [jobName release];
  jobName = nil;
  [jobDesc release];
  jobDesc = nil;
  [self setDefaultCostCentre:nil];
  [self setCostCentrePicker:nil];
  [super viewDidUnload];
  // Release any retained subviews of the main view.
  // e.g. self.myOutlet = nil;
}

#pragma mark - Added methods (did not exist in original template code)

- (id)init {
  [super initWithNibName:nil bundle:nil];
  // Set up navigation items
  UIBarButtonItem *bbi;
  
  // Done item
  bbi = [[UIBarButtonItem alloc]
         initWithBarButtonSystemItem:UIBarButtonSystemItemDone 
         target:self
         action:@selector(create:)];
  [[self navigationItem]setRightBarButtonItem:bbi];
  [bbi release];
  
  // Cancel item
  bbi = [[UIBarButtonItem alloc] 
         initWithBarButtonSystemItem:UIBarButtonSystemItemCancel 
         target:self 
         action:@selector(cancel:)];
  
  // Get a list of cost centres from core data via the application delegate
  iTimesheetAppDelegate *ad = [iTimesheetAppDelegate sharedAppDelegate];
  NSArray *list = [ad allInstancesOf:@"CostCentre" orderedBy:@"name"];
  [self setCostCentres:[list mutableCopy]];
  
  return self;
}

- (void)dealloc
{
  [costCentres release];
  [jobCostCentre release];
  [project release];
  [jobName release];
  [jobNameValue release];
  [jobDesc release];
  [jobDescValue release];
  [defaultCostCentre release];
  [costCentrePicker release];
  [super dealloc];
}

- (void)viewDidAppear:(BOOL)animated {
  [super viewDidAppear:animated];
  // Make the keyboard appear
  [jobName becomeFirstResponder];
}

- (IBAction)textFieldDidEndOnExit:(id)sender {
  // Make the keyboard disappear
  [sender resignFirstResponder];
}

- (IBAction)create:(id)sender {
  // Set values provided by user
  [self setJobNameValue:[jobName text]];
  [self setJobDescValue:[jobDesc text]];
  // Get the currently selected cost centre
  NSUInteger row = [[self costCentrePicker] 
                    selectedRowInComponent:kCostCentreComponent];
  // Get the cost centre associated with that row
  PPCostCentre *cc = [[self costCentres] objectAtIndex:row];
  // Set the job cost centre from the cost centre name
  [self setJobCostCentre:cc];
  // Slide back to the previous view controller
  [[self navigationController] popViewControllerAnimated:YES];
}

- (IBAction)cancel:(id)sender {
  // Clear out any values set by the user
  [self setJobNameValue:nil];
  [self setJobDescValue:nil];
  [self setJobCostCentre:nil];
  // Slide back to the previous view controller
  [[self navigationController] popViewControllerAnimated:YES];
}

- (void)setCostCentre:(NSString *)costCentre {
  for (PPCostCentre *cc in costCentres) {
    NSLog(@"%@ = %@", [cc name], costCentre);
    if ([[cc name] compare:costCentre] == 0) {
      jobCostCentre = cc;
    }
  }
}

// Picker Datasource Methods

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView {
  // We only have one component in the picker for our cost centres
  return 1;
}

- (NSInteger)pickerView:(UIPickerView *)pickerView 
numberOfRowsInComponent:(NSInteger)component {
  // We want one entry in the picker for each cost centre, so return the count 
  // of cost centres in our array.
  return [[self costCentres] count];
}

// Picker Delegate Methods
- (NSString *)pickerView:(UIPickerView *)pickerView
             titleForRow:(NSInteger)row 
            forComponent:(NSInteger)component {
  // We want to display the cost centre name in our picker.
  // Get the cost centre from our array using the passed row as our index
  PPCostCentre *cc = [[self costCentres] objectAtIndex:row];
  
  // Return the cost centre name.
  return [cc name];
}

@end
