//
//  InsertProjectViewController.m
//  iTimesheet
//
//  Created by Mark Brindle on 08/04/2011.
//  Copyright 2011 ARKEMM Software Limited. All rights reserved.
//

#import "InsertProjectViewController.h"
// Added to allow us access to the application delegate
#import "iTimesheetAppDelegate.h"
// Added to handle core data cost centre details
#import "PPCostCentre.h"

@implementation InsertProjectViewController

// These were automatically added by XCode when we created the outlets
@synthesize nameField;
@synthesize descField;
@synthesize costCentrePicker;

// We need to explicitly synthesize our manually added properties
@synthesize projectName;
@synthesize projectDesc;
@synthesize projectCostCentre;
@synthesize costCentres;

#pragma mark - boilerplate code (provided by template)

- (void)didReceiveMemoryWarning
{
  // Releases the view if it doesn't have a superview.
  [super didReceiveMemoryWarning];
  
  // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
  // Return YES for supported orientations
  return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Existing methods (in the original template) that were amended

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
  // Just call init on ourselves.  We'll do all the work there.  Doing this 
  // ensures that we get properly initialised regardless of which mechanism is
  // used.  It only works because we are ignoring the parameters passed in.
  return [self init];
}

- (void)dealloc
{
  // These were automatically added by XCode when we created the outlets
  [nameField release];
  [descField release];
  [costCentrePicker release];
  
  // Release memory associated with our manually created properties
  [costCentres release];
  [projectCostCentre release];
  [projectDesc release];
  [projectName release];
  
  [super dealloc];
}

- (void)viewDidLoad
{
  [super viewDidLoad];
  // Do any additional setup after loading the view from its nib.

  // Get a list of cost centres from core data via the application delegate
  iTimesheetAppDelegate *ad = [iTimesheetAppDelegate sharedAppDelegate];
  NSArray *list = [ad allInstancesOf:@"CostCentre" orderedBy:@"name"];
  [self setCostCentres:[list mutableCopy]];
  
  [nameField setAutocorrectionType:UITextAutocorrectionTypeNo];
  [descField setText:nil];
}

- (void)viewDidUnload
{
  // Reset our manually created properties
  [self setProjectName:nil];
  [self setProjectDesc:nil];
  [self setProjectCostCentre:nil];
  [self setCostCentres:nil];
  
  // These were automatically added by XCode when we created the outlets
  [self setNameField:nil];
  [self setDescField:nil];
  [self setCostCentrePicker:nil];
  [super viewDidUnload];

  // Release any retained subviews of the main view.
  // e.g. self.myOutlet = nil;
}

#pragma mark - Added methods (did not exist in original template code)

- (id)init {
  // Call our parent initialiser
  [super initWithNibName:nil bundle:nil];
  
  // Do our own setup now
  [self setTitle:@"New Project"];
  
  // Set up navigation items
  UIBarButtonItem *bbi;
  
  // Done item
  bbi = [[UIBarButtonItem alloc]
         initWithBarButtonSystemItem:UIBarButtonSystemItemDone 
         target:self
         action:@selector(create:)];
  [[self navigationItem]setRightBarButtonItem:bbi];
  [bbi release];
  
  // Cancel item
  bbi = [[UIBarButtonItem alloc] 
         initWithBarButtonSystemItem:UIBarButtonSystemItemCancel 
         target:self 
         action:@selector(cancel:)];
  
  return self;
}

- (IBAction)create:(id)sender {
  // Set our property values from details provided by the user
  [self setProjectName:[nameField text]];
  [self setProjectDesc:[descField text]];
  
  // Get the cost centre that the user selected in the picker
  // First get the currently selected row from the picker
  NSUInteger row = [[self costCentrePicker]
                    selectedRowInComponent:kPickerComponent];
  // Now get the cost centre associated with that row
  PPCostCentre *cc = [[self costCentres] objectAtIndex:row];
  // Finally assign the name of this cost centre to the project
  [self setProjectCostCentre:[cc name]];
  
  // Slide back to the previous view controller
  [[self navigationController] popViewControllerAnimated:YES];
}

- (IBAction)cancel:(id)sender {
  // Clear out any values set by the user
  [self setProjectName:nil];
  [self setProjectDesc:nil];
  [self setProjectCostCentre:nil];
  
  // Slide back to the previous view controller
  [[self navigationController] popViewControllerAnimated:YES];
}

- (void)viewDidAppear:(BOOL)animated {
  [super viewDidAppear:animated];

  // Make the keyboard appear as focus will be on the name text field when
  // we enter the view.
  [nameField becomeFirstResponder];
}

// Picker Datasource Methods

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView {
  // There is only one revolving list of cost centres in this case.
  return 1;
}

- (NSInteger)pickerView:(UIPickerView *)pickerView 
numberOfRowsInComponent:(NSInteger)component {
  // We need to tell the picker how many entries to expect.  In this case we 
  // want one entry for each cost centre that was retrieved from core data and
  // stored in our costCentres array; so return the array count.
  return [[self costCentres] count];
}

// Picker Delegate Methods
- (NSString *)pickerView:(UIPickerView *)pickerView
             titleForRow:(NSInteger)row 
            forComponent:(NSInteger)component {
  // Called when the picker needs to get a string value representation to
  // display in the picker component row.
  
  // Find the cost centre associated with the row in our picker.  The row acts
  // as an index into our costCentres array; so get the object at that row.
  PPCostCentre *cc = [[self costCentres] objectAtIndex:row];
  
  // having found the cost centre object; we want to display its name; so 
  // return that.
  return [cc name];
}

// Text field methods

- (IBAction)textFieldDoneEditing:(id)sender {
  // Called whenever a text field loses focus.  Calling resignFirstResponder on
  // the sender will cause it to hide the keyboard.
  [sender resignFirstResponder];
}

@end
