//
//  JobTableViewController.m
//  iTimesheet
//
//  Created by Mark Brindle on 03/04/2011.
//  Copyright 2011 ARKEMM Software Limited. All rights reserved.
//

#import "JobTableViewController.h"

// Added so we can get a reference to our App delegate
#import "iTimesheetAppDelegate.h"

// Added to access core data objects & categories
#import "PPTimecard.h"
#import "PPJob.h"
#import "PPJob-TimeSpent.h"
#import "PPJob-Timecard.h"
#import "PPProject.h"
#import "PPProject-SortedJobs.h"
#import "PPCostCentre.h"

// Added so we can navigate to other views
#import "InsertJobViewController.h"
#import "TimecardTableViewController.h"
#import "ProjectJobTableViewCell.h"

@implementation JobTableViewController

// Synthesize our manually added properties
@synthesize jobs;
@synthesize project;

#pragma mark - boilerplate code (provided by template)

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/

/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
}
*/

/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/

#pragma mark - Existing methods (in the original template) that were amended

- (void)dealloc 
{
  // Release memory associated with our manually added properties
  [project release];
  [jobs release];
  
  [super dealloc];
}

- (void)viewDidLoad
{
  [super viewDidLoad];

  self.clearsSelectionOnViewWillAppear = NO;
  
  // Add a title for this view
  [self setTitle:[NSString stringWithFormat:@"Jobs"]];
 
  // Add an Apple standard 'Add' button (Plus sign) on the right side of the
  // view navigator
  UIBarButtonItem *addButton =
  [[UIBarButtonItem alloc] 
   initWithBarButtonSystemItem:UIBarButtonSystemItemAdd 
   target:self
   action:@selector(insertNewJob)];
  self.navigationItem.rightBarButtonItem = addButton;
  [addButton release];
  
  // Get a list of jobs from the project and keep in our jobs array
  NSArray *list = [project sortedJobs];
  [self setJobs:[list mutableCopy]];

}

- (void)viewWillAppear:(BOOL)animated {
  [super viewWillAppear:animated];
  // Set the title to reflect the project (if we have one)
  if ([project name] != nil) {
    [self setTitle:[NSString stringWithFormat:@"%@ jobs", [project name]]];
  }
  
  // Are we returning from InsertJobViewController?
  if (jvc) {
    // We are coming back to this view after adding a new job to our project
    NSString *jobName = [jvc jobNameValue];
    // Do we have a name for the new job?
    if ([jobName length] > 0) {
      // We have a valid new job; so insert as a managed object into core data.
      iTimesheetAppDelegate *ad = [iTimesheetAppDelegate sharedAppDelegate];
      NSManagedObjectContext *moc = [ad managedObjectContext];
      PPJob *job = [NSEntityDescription 
                               insertNewObjectForEntityForName:@"Job"
                               inManagedObjectContext:moc];
      
      // Get the new job's details from our insertJobViewController.
      [job setName:jobName];
      [job setDesc:[jvc jobDescValue]];
      [job setProject:project];
      [job setHourlyRate:[[jvc jobCostCentre] hourlyRate]];
      [job setTimeSpent:[NSNumber numberWithInt:0]];
      
      // Add to our array of jobs
      [jobs addObject:job];
      
      // Re-sort the array
      NSSortDescriptor *sd = [[NSSortDescriptor alloc] initWithKey:@"name"
                                                         ascending:YES];
      NSArray *sds = [NSArray arrayWithObject:sd];
      [sd release];
      [jobs sortUsingDescriptors:sds];
      
      // Re-display the table view
      [[self tableView] reloadData];
    }
    
    // We've finished with the insertJobViewTable so release memory associated 
    // with it & reset for future use
    [jvc release];
    jvc = nil;
  }
  
  // Clear the selection
  NSIndexPath *selectedPath = [[self tableView] indexPathForSelectedRow];
  if (selectedPath) {
    [[self tableView] deselectRowAtIndexPath:selectedPath animated:NO];
  }
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // We are not grouping jobs in any way, so we only have one section.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // The number of rows to populate is the same as the number of jobs we have.
    return [self.jobs count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView 
         cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  // An identifier that we use to look for any cells we can reuse
  static NSString *CellIdentifier = @"JobCellIdentifier";
  
  // See if any cells with our identifier have been dequeued & can be reused 
  ProjectJobTableViewCell *cell = 
    (ProjectJobTableViewCell *)[tableView 
                              dequeueReusableCellWithIdentifier:CellIdentifier];
  
  // We didn't get a cell to reuse, so we'll have to create one instead
  if (cell == nil) {
    NSArray *nib = [[NSBundle mainBundle] 
                    loadNibNamed:@"ProjectJobTableViewCell" 
                    owner:self 
                    options:nil];
    for (id oneObject in nib) {
      if ([oneObject isKindOfClass:[ProjectJobTableViewCell class]]) {
        cell = (ProjectJobTableViewCell *)oneObject;
      }
    }
  }
  
  // Get job details to display using the row as an index into our jobs array.
  PPJob *job = [[self jobs] objectAtIndex:[indexPath row]];
  
  // Set the cell to display details for our job
  cell.jobLabel.text = [job name];
  cell.jobDesc.text = [job desc];
  cell.jobHours.text = [job timeSpentText];
  if ([job hasTimecards] > 0) {
    // Work out whether user is stopping an existing timecard, or creating one.
    if ([[job lastTimecard] end] == nil) {
      cell.jobStatusImage.image = [UIImage imageNamed:@"Stop.png"];
    } else {
      cell.jobStatusImage.image = [UIImage imageNamed:@"Go.png"];
    }
    // We have timecard detail data so set a detail disclosure button
    [cell setAccessoryType:UITableViewCellAccessoryDetailDisclosureButton];
  } else {
    // We have no timecard detail data yet so set up to start one.
    cell.jobStatusImage.image = [UIImage imageNamed:@"Go.png"];
  }
  
  // Pass back the populated cell
  return cell;
}

// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
  if (editingStyle == UITableViewCellEditingStyleDelete) {
    // Delete the job & row from the data source
    [self removeJob:[indexPath row]];
    [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] 
                     withRowAnimation:UITableViewRowAnimationFade];
  }   
//    else if (editingStyle == UITableViewCellEditingStyleInsert) {
//        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
//    }   
}

#pragma mark - Table view delegate

-(void)tableView:(UITableView *)tableView 
didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
  // Called when cell is tapped to start / stop a timesheet entry
  PPJob *job = [[self jobs] objectAtIndex:[indexPath row]];
  if ([job hasTimecards]) {
    // Get last timecard
    PPTimecard *lastTimecard = [job lastTimecard];
    if ([lastTimecard end] == nil) {
      NSDate *now = [[NSDate alloc] init];
      [lastTimecard setEnd:now];
      [now release];
      NSTimeInterval jobSeconds = [[lastTimecard end] 
                                     timeIntervalSinceDate:[lastTimecard start]];
      [job setTimeSpent:[NSNumber numberWithDouble:[[job timeSpent] doubleValue]
                         + jobSeconds]];
    } else {
      // Create a new Timecard
      [self createTimecard:job];
    }
  } else {
    // Create a new Timecard
    [self createTimecard:job];
  }
  [[self tableView] reloadData];
}

#pragma mark - Added methods (did not exist in original template code)

-(CGFloat)tableView:(UITableView *)tableView
heightForRowAtIndexPath:(NSIndexPath *)indexPath {
  // Return our jobTableViewCell height
  return kTableViewRowHeight;
}

- (void)createTimecard:(PPJob *)job {
  // Create a new timecard via the core data managed object context.
  iTimesheetAppDelegate *ad = [iTimesheetAppDelegate sharedAppDelegate];
  NSManagedObjectContext *moc = [ad managedObjectContext];
  PPTimecard *tc = [NSEntityDescription 
                    insertNewObjectForEntityForName:@"Timecard"
                    inManagedObjectContext:moc];
  
  // Set the new timecard details
  [tc setJob:job];
  
  // Create a date object with the current time & set our timecard start
  NSDate *now = [[NSDate alloc] init];
  [tc setStart:now];
  
  // Release memory associated with our date object as we've finished with it.
  [now release];
}

-(void)tableView:(UITableView *)tableView 
accessoryButtonTappedForRowWithIndexPath:(NSIndexPath *)indexPath {
  // Called when disclosure button is tapped to display Timecards.  Create a
  // timecard view controller.
  TimecardTableViewController *vc = [[TimecardTableViewController alloc] init];
  
  // Give the view controller a reference to our job 
  [vc setJob:[jobs objectAtIndex:[indexPath row]]];
  
  // Push the view controller onto our navigation controllers stack
  [[self navigationController] pushViewController:vc animated:YES];
  
  // Release our interest in memory associated with the view controller as we're 
  // no longer concerned with it once we've handed it off.
  [vc release];
}

-(void)insertNewJob {
  // Create an instance of insertJobViewController, give it a reference to the 
  // cost centre associated with our project and push it onto our navigation
  // controllers stack.
  jvc = [[InsertJobViewController alloc] init];
  [jvc setCostCentre:[project costCentre]];
  [[self navigationController] pushViewController:jvc animated:YES];
}

-(void) removeJob:(NSUInteger)index {
  // Get the job associated with the index provided from our array of jobs.
  PPJob *job = [jobs objectAtIndex:index];
  
  // Tell the core data managed object context to delete this job.
  iTimesheetAppDelegate *ad = [iTimesheetAppDelegate sharedAppDelegate];
  NSManagedObjectContext *moc = [ad managedObjectContext];
  [moc deleteObject:job];
  
  // Remove the job from our array to keep things in synch.
  [jobs removeObjectAtIndex:index];
}

@end
